Attribute VB_Name = "Shared"
Option Explicit

Public Type GamePref       ' game preferences saved in high score and game files
    MajVer As Integer       ' program major version
    MinVer As Integer       ' program minor version
    Diff As Integer         ' game difficulty
    Size As Integer         ' board size
    Pieces As Integer       ' game pieces
    Sound As Boolean        ' sound toggle
    ACount As Boolean       ' automatic count toggle
    AFind As Boolean        ' automatic find largest group toggle
    XPos As Single          ' gamewindow x position
    YPos As Single          ' gamewindow y position
    CountXPos As Single     ' count window x
    CountYPos As Single     ' count window y
    TotGames As Long        ' total game count
    OneClick As Boolean     ' one click mode
    DemoAuto As Boolean     ' automatic demo mode toggle
    DemoSpeed As Integer    ' demo mode speed
    Concentr As Boolean     ' concentration mode
    ConcTime As Integer     ' time before hiding pieces
    RegNum As Long          ' registration number
    RegName As String       ' registration name
    SaveName As String      ' last high score entry name
    Zoom As Boolean         ' window zoomed to fit screen
    Trans As Boolean        ' transparent background
    BGColor As Double       ' background color
    SndRemove As Boolean    ' use (true) wav file, or (false) built-in sound
    SndNew As Boolean       ' new game sounds
    SndUndo As Boolean      ' undo sound
    SndNoUndo As Boolean    ' no undo sound
    SndHS As Boolean        ' new high score sound
    SndSetup As Boolean     ' setup sound
    SndSetupClick As Boolean
    SndGreeting As Boolean  ' welcome to game sound
    SndBounce As Boolean    ' bounce sound
    End Type

Public Status As GamePref

Public Board(1 To 15, 1 To 30) As Integer       ' current game board
Public Selected(1 To 15, 1 To 30) As Boolean    ' board with selected pieces
Public PieceCount(1 To 7) As Integer            ' count of each type piece

Public QResponse As String          ' response from frmYN (button clicked)
Private SoundStr As String          ' piece removal sound
Private NoUndoSoundStr As String    ' no undos available sound
Private UndoSoundStr As String      ' undo sound
Private NewSoundStr As String       ' new game sound
Private HSSoundStr As String        ' new high score sound
Private SetupSoundStr As String     ' enter setup mode sound
Private SetupClickSoundStr As String     ' setup click sound
Private BounceSoundStr As String    ' about box bounce sound
Private GreetingSoundStr As String   ' welcome to game sound

' used for mouse cursor
Public Type POINTAPI
    X As Long
    Y As Long
End Type


' sound functions
Private Declare Function sndPlaySound Lib "winmm.dll" Alias "sndPlaySoundA" _
    (ByVal lpszSoundName As Any, ByVal uFlags As Long) As Long
Private Declare Function waveOutGetNumDevs Lib "winmm.dll" () As Long

' web browser interaction
Private Declare Function ShellExecute Lib "shell32.dll" Alias "ShellExecuteA" _
  (ByVal hwnd As Long, ByVal lpOperation As String, ByVal lpFile As String, _
   ByVal lpParameters As String, ByVal lpDirectory As String, ByVal nShowCmd As Long) As Long

' directory searching
Private Directory As String
Private Const INVALID_HANDLE_VALUE = -1
Private Const MAX_PATH = 260

Private Type FILETIME
   dwLowDateTime As Long
   dwHighDateTime As Long
End Type

Private Type WIN32_FIND_DATA
   dwFileAttributes As Long
   ftCreationTime As FILETIME
   ftLastAccessTime As FILETIME
   ftLastWriteTime As FILETIME
   nFileSizeHigh As Long
   nFileSizeLow As Long
   dwReserved0 As Long
   dwReserved1 As Long
   cFileName As String * MAX_PATH
   cAlternate As String * 14
End Type

Private Declare Function FindFirstFile Lib "kernel32" Alias "FindFirstFileA" _
   (ByVal lpFileName As String, lpFindFileData As WIN32_FIND_DATA) As Long

Private Declare Function FindClose Lib "kernel32" (ByVal hFindFile As Long) As Long
Public Declare Function SendMessage Lib "user32" Alias "SendMessageA" (ByVal hwnd As Long, ByVal wMsg As Long, ByVal wParam As Long, lParam As Any) As Long
Public Declare Function ReleaseCapture Lib "user32" () As Long


Public Sub LoadSounds()
Status.SndRemove = False
If GameWindow.GamePieces <= 0 Then
    SoundStr = StrConv(LoadResData(101 + GameWindow.GamePieces, "SOUND"), vbUnicode)
Else
Dim FileName As String
FileName = CurDir & "\" & GameWindow.GamePiecesName & "\sound.wav"
    If FileExists(FileName) Then
        SoundStr = FileName
        Status.SndRemove = True
    Else    'use default block sound
        SoundStr = StrConv(LoadResData(101, "SOUND"), vbUnicode)
    End If
End If

Status.SndNoUndo = False
NoUndoSoundStr = "noundo.wav"
If FileExists(NoUndoSoundStr) Then
    Status.SndNoUndo = True
Else
    NoUndoSoundStr = StrConv(LoadResData(200, "SOUND"), vbUnicode)
End If

Status.SndUndo = False
UndoSoundStr = "undo.wav"
If FileExists(UndoSoundStr) Then
    Status.SndUndo = True
Else
    UndoSoundStr = StrConv(LoadResData(201, "SOUND"), vbUnicode)
End If

Status.SndNew = False
NewSoundStr = "newgame.wav"
If FileExists(NewSoundStr) Then
    Status.SndNew = True
Else
    NewSoundStr = StrConv(LoadResData(202, "SOUND"), vbUnicode)
End If

Status.SndHS = False
HSSoundStr = "highscore.wav"
If FileExists(HSSoundStr) Then
    Status.SndHS = True
Else
    HSSoundStr = StrConv(LoadResData(203, "SOUND"), vbUnicode)
End If

Status.SndSetup = False
SetupSoundStr = "setup.wav"
If FileExists(SetupSoundStr) Then
    Status.SndSetup = True
Else
    SetupSoundStr = StrConv(LoadResData(101, "SOUND"), vbUnicode)
End If

Status.SndSetupClick = False
SetupClickSoundStr = "setupclick.wav"
If FileExists(SetupClickSoundStr) Then
    Status.SndSetupClick = True
Else
    SetupClickSoundStr = StrConv(LoadResData(101, "SOUND"), vbUnicode)
End If

Status.SndBounce = False
BounceSoundStr = "bounce.wav"
If FileExists(BounceSoundStr) Then
    Status.SndBounce = True
Else
    BounceSoundStr = StrConv(LoadResData(101, "SOUND"), vbUnicode)
End If

Status.SndGreeting = False
GreetingSoundStr = "greeting.wav"
If FileExists(GreetingSoundStr) Then
    Status.SndGreeting = True
Else
    GreetingSoundStr = "" ' StrConv(LoadResData(301, "SOUND"), vbUnicode)
End If
End Sub

Public Sub PlaySound()
If Not GameWindow.GameSound Then Exit Sub
If GameWindow.BeepOnly Then
    Beep
Else
'    If GameWindow.GamePieces > 0 Then sndPlaySound SoundStr, &H2 _

    If Status.SndRemove Then sndPlaySound SoundStr, &H2 _
    Else sndPlaySound SoundStr, &H2 Or &H4
End If
DoEvents
End Sub

Public Sub PlayASound(ByVal what As String)
'Dim SoundBuffer As String
'SoundBuffer = StrConv(LoadResData(ResourceId, "Sound"), vbUnicode)
'Ret = sndPlaySound(SoundBuffer, SND_ASYNC Or SND_NODEFAULT Or SND_MEMORY)
If Not GameWindow.GameSound Then Exit Sub
'Dim Ret As Variant
If GameWindow.BeepOnly Then
    Beep
Else
    If Not GameWindow.DemoMode Then
    Select Case what
    Case "noundo"
        If Status.SndNoUndo Then
            sndPlaySound NoUndoSoundStr, &H1
        Else
            sndPlaySound NoUndoSoundStr, &H1 Or &H2 Or &H4
            End If
    
    Case "undo"
        If Status.SndUndo Then
            sndPlaySound UndoSoundStr, &H1
        Else
            sndPlaySound UndoSoundStr, &H1 Or &H2 Or &H4
            End If
    
    Case "new"
        If Status.SndNew Then
            sndPlaySound NewSoundStr, &H1
        Else
            sndPlaySound NewSoundStr, &H1 Or &H2 Or &H4
            End If
    
    Case "high"
        If Status.SndHS Then
            sndPlaySound HSSoundStr, &H1
        Else
            sndPlaySound HSSoundStr, &H1 Or &H2 Or &H4
            End If
    
    Case "setup"
        If Status.SndSetup Then
            sndPlaySound SetupSoundStr, &H1
        Else
            sndPlaySound SetupSoundStr, &H1 Or &H2 Or &H4
            End If
    
    Case "setupclick"
        If Status.SndSetupClick Then
            sndPlaySound SetupClickSoundStr, &H1
        Else
            sndPlaySound SetupClickSoundStr, &H1 Or &H2 Or &H4
            End If
    
    Case "bounce"
        If Status.SndBounce Then
            sndPlaySound BounceSoundStr, &H1
        Else
            sndPlaySound BounceSoundStr, &H1 Or &H2 Or &H4
            End If
    
    Case "greeting"
        If Status.SndGreeting Then
            sndPlaySound GreetingSoundStr, &H1
        Else
            'sndPlaySound GreetingSoundStr, &H1 Or &H2 Or &H4
            End If
    Case Else

    End Select
    End If
End If
DoEvents
End Sub

Public Function SoundCardInstalled() As Boolean
     SoundCardInstalled = waveOutGetNumDevs > 0
End Function

Function ShellToBrowser%(frm As Form, ByVal URL$, ByVal WindowStyle%)
    Dim api%
    Dim temp As Integer
        api% = ShellExecute(frm.hwnd, "open", URL$, "", App.Path, WindowStyle%)
    'Check return value
    If api% < 31 Then
        'error code - see api help for more info
        temp = MsgBox(App.Title & " had a problem running your web browser." & vbCrLf & "You should check that your browser is correctly installed." & vbCrLf & "(Error" & Format$(api%) & ")", 48, "Browser Unavailable")
        ShellToBrowser% = False
    ElseIf api% = 32 Then
        'no file association
        MsgBox App.Title & " could not find a file association for " & _
          URL$ & " on your system. You should check that your browser" & _
          "is correctly installed and associated with this type of file.", 48, "Browser Unavailable"
        ShellToBrowser% = False
    Else        'It worked!
        ShellToBrowser% = True
    End If
End Function

Public Function GetVersionNum() As String
Dim Ver1 As String
Dim Ver2 As String
Dim Ver3 As String

Ver1 = CStr(App.Major)
If App.Major < 1 Then Ver1 = "0"

Ver2 = "." & CStr(App.Minor)
If App.Minor >= 10 Then Ver2 = "." & CStr(App.Minor)
If App.Minor < 10 Then Ver2 = ".0" & CStr(App.Minor)
If App.Minor < 1 Then Ver2 = ""

Ver3 = ""
If App.Revision >= 10 Then Ver3 = "" & CStr(App.Revision)
If App.Revision < 10 Then Ver3 = "0" & CStr(App.Revision)
If App.Revision < 1 Then Ver3 = ""

GetVersionNum = Ver1 & Ver2 & Ver3
End Function

Public Sub SearchDirectories()
Dim i As Integer        'loop counter
Dim j As Integer        ' loop counter
Dim FileName As String  'inf filenames
Dim DirName As String
Dim Contents() As String    'meta contents of the directory
'Dim complete As String  'complete string in text box
Dim Valid As Boolean    ' valid directory and contents

With GameWindow
'.Dir1.Path = .Drive1.Drive
DoEvents
Screen.MousePointer = vbHourglass

For i = 0 To .Dir1.ListCount - 1
Valid = True

For j = 1 To 7
    FileName = .Dir1.List(i) & "\0" & j & ".bmp"
    If Not FileExists(FileName) Then Valid = False
    DoEvents
Next j

For j = 11 To 17
    FileName = .Dir1.List(i) & "\" & j & ".bmp"
    If Not FileExists(FileName) Then Valid = False
    DoEvents
Next j

If Valid Then
' add a menu item for the new pieces
' set the menu item name to match the folder name
.NumPieceTypes = .NumPieceTypes + 1
Contents = Split(.Dir1.List(i), "\")
j = UBound(Contents)
DirName = Contents(j)

Load .mnuPiece(.NumPieceTypes)
.mnuPiece(.NumPieceTypes).Caption = DirName
.mnuPiece(.NumPieceTypes).Checked = False
'.mnuPiece(.NumPieceTypes).Visible = True
End If

Next i
End With
Screen.MousePointer = vbDefault
End Sub

Public Function FileExists(ByVal Source As String) As Boolean
Dim WFD As WIN32_FIND_DATA
Dim hFile As Long

hFile = FindFirstFile(Source, WFD)
FileExists = hFile <> INVALID_HANDLE_VALUE

Call FindClose(hFile)
End Function

